-- **************************************************************************
--   TitanPalsConfig.lua
--
--   By: KanadiaN
--        (Lindarena@Laughing Skull)
-- **************************************************************************

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Titan", true );
local LB = LibStub( "AceLocale-3.0" ):GetLocale( "TitanPals", true );
local AceConfigDialog = LibStub( "AceConfigDialog-3.0" );
local AceConfig = LibStub( "AceConfig-3.0" );

-- **************************************************************************
-- NAME : Core Vars
-- DESC : These are here so they do not get changed like the locales would
-- **************************************************************************

palsDB = nil;
TitanPalsFormatToRemove = LB["TITAN_PALS_CONFIG_NONE"];

local PlugInName = "TitanPals";
LB["TITAN_PALS_CORE_ID"] = "Pals";
LB["TITAN_PALS_CORE_VERSION"] = tonumber( GetAddOnMetadata( PlugInName, "Version" ) );
LB["TITAN_PALS_CORE_DTAG"] = "|cffff0000<TitanPals Debug>|r %s";
LB["TITAN_PALS_CORE_ITAG"] = "|cffff9900<TitanPals>|r %s";
LB["TITAN_PALS_CORE_ETAG"] = "|cffff9900<TitanPals Error>|r %s";
LB["TITAN_PALS_CORE_DBVER"] = tonumber( 2 );
LB["TITAN_PALS_CORE_MINLEVEL"] = tonumber( 0 );
LB["TITAN_PALS_CORE_MAXLEVEL"] = tonumber( 85 );
LB["TITAN_PALS_CORE_ARTWORK"] = "Interface\\FriendsFrame\\UI-Toast-ChatInviteIcon";
LB["TITAN_PALS_CORE_DEBUG_HEADER"] = "Debugging"
LB["TITAN_PALS_CORE_DEBUG_EVENT"] = "Events"
LB["TITAN_PALS_CORE_DEBUG_EVENT_DESC"] = "Displays event as they happen in the chat frame"
LB["TITAN_PALS_CORE_DEBUG_LOGEVENT"] = "Log Event"
LB["TITAN_PALS_CORE_DEBUG_LOGEVENT_DESC"] = "This will log the events as they happen for later veiwing"
LB["TITAN_PALS_CORE_DEBUG_HEADER_PRINT"] = "Print Functions"
LB["TITAN_PALS_CORE_DEBUG_PRINT_ALTS"] = "Print Alts"
LB["TITAN_PALS_CORE_DEBUG_PRINT_ALTS_DESC"] = "This will print a list of your alts to the chat frame"
LB["TITAN_PALS_CORE_DEBUG_PRINT_FRIENDS"] = "Print Friends"
LB["TITAN_PALS_CORE_DEBUG_PRINT_FRIENDS_DESC"] = "This will print a list of your friends to the chat frame"
LB["TITAN_PALS_CORE_DEBUG"] = "Debug"
LB["TITAN_PALS_CORE_DEBUG_DESC"] = "Enable Debug Mode"
LB["TITAN_PALS_CORE_DEBUG_REMAUTH"] = "Remove Author"
LB["TITAN_PALS_CORE_DEBUG_REMAUTH_DESC"] = "Removes |cffff0000%s|r as a Addon Author"
LB["TITAN_PALS_CORE_ADDAUTHOR"] = "|cff00ff00%s|r Has been added as a Author"
LB["TITAN_PALS_CORE_REMAUTHOR"] = "|cffff0000%s|r Has been removed as a Author"
LB["TITAN_PALS_HELP_ADDAUTH"] = "  |cffff0000/%s AddAuth|r - Add a AddOn Author to Data Array"
LB["TITAN_PALS_HELP_REMAUTH"] = "  |cffff0000/%s RemAuth|r - Remove a AddOn Author from the Data Array"


-- **************************************************************************
-- NAME : TitanPanelPals_ConfigUtils(a)
-- DESC : Pull information from .toc for config dialog
-- **************************************************************************
local function TitanPanelPals_ConfigUtils( a )
     if ( a == "GetAuthor" ) then
          return GetAddOnMetadata( PlugInName, "Authors" );
     elseif ( a == "GetTranslation" ) then
          return GetAddOnMetadata( PlugInName, "X-Translation" );
     elseif ( a == "GetCategory" ) then
          return GetAddOnMetadata( PlugInName, "X-Category" );
     elseif ( a == "GetEmail" ) then
          return GetAddOnMetadata( PlugInName, "X-Email" );
     elseif ( a == "GetWebsite" ) then
          return GetAddOnMetadata( PlugInName, "X-Website" );
     elseif ( a == "GetVersion" ) then
          return tostring( GetAddOnMetadata( PlugInName, "Version" ) );
     end
end

-- **************************************************************************
-- NAME : PalsOpt
-- DESC : This is the root of the config dialog
-- **************************************************************************
local PalsOpt = {
     type = "group",
     name = LB["TITAN_PALS_ADDON_LABEL"],
     args = {
          confgendesc = {
               type = "description", order = 1,
               name = LB["TITAN_PALS_ABOUT_DESC"],
               cmdHidden = true
          },
	  cat1 = {

               type = "group", order = 2, inline = true,
               name = LB["TITAN_PALS_ABOUT"],
               args = {
                    confversiondesc = {
                         type = "description", order = 1,
                         name = LB["TITAN_PALS_ABOUT_VERSION"]:format( TitanPanelPals_ConfigUtils( "GetVersion" ) ),
                         cmdHidden = true
                    },
                    confauthordesc = {
                         type = "description", order = 2,
                         name = LB["TITAN_PALS_ABOUT_AUTHOR"]:format( TitanPanelPals_ConfigUtils( "GetAuthor" ) ),
                         cmdHidden = true
                    },
                    conflangauthordesc = {
                         type = "description", order = 3,
                         name = LB["TITAN_PALS_ABOUT_TRANSLATION"]:format( TitanPanelPals_ConfigUtils( "GetTranslation" ) ),
                         cmdHidden = true
                    },
                    confcatdesc = {
                         type = "description", order = 4,
                         name = LB["TITAN_PALS_ABOUT_CATEGORY"]:format( TitanPanelPals_ConfigUtils( "GetCategory" ) ),
                         cmdHidden = true
                    },
                    confemaildesc = {
                         type = "description", order = 5,
                         name = LB["TITAN_PALS_ABOUT_EMAIL"]:format( TitanPanelPals_ConfigUtils( "GetEmail" ) ),
                         cmdHidden = true
                    },
                    confwebsitedesc = {
                         type = "description", order = 6,
                         name = LB["TITAN_PALS_ABOUT_WEBSITE"]:format( TitanPanelPals_ConfigUtils( "GetWebsite" ) ),
                         cmdHidden = true
                    },
                    confmemorydesc = {
                         type = "description", order = 7,
                         name = function()
                              UpdateAddOnMemoryUsage();
                              mem = floor( GetAddOnMemoryUsage( PlugInName ) );
                              return LB["TITAN_PALS_ABOUT_MEMORY"]:format( mem );
                         end,
                         cmdHidden = true,
                    },
               },
	  },
	  cat2 = {
	       type = "group", order = 3,
	       name = LB["TITAN_PALS_CONFIG_HEADER_SETTINGS"],
	       args = {
                    tooltipshowignored = {
                         type = "toggle", order = 1,
                         name = LB["TITAN_PALS_CONFIG_SHOWIGNORED"],
                         desc = LB["TITAN_PALS_CONFIG_SHOWIGNORED_DESC"],
                         get = function() return TitanPals.Settings.ShowIgnored; end,
                         set = function( _, value )
                              TitanPals.Settings.ShowIgnored = value;
                              TitanPanelButton_UpdateButton( LB["TITAN_PALS_CORE_ID"] );
                         end,
                    },
                    tooltipshowmem = {
                         type = "toggle", order = 2,
                         name = LB["TITAN_PALS_CONFIG_SHOWMEM"],
                         desc = LB["TITAN_PALS_CONFIG_SHOWMEM_DESC"],
                         get = function() return TitanPals.Settings.ShowMem; end,
                         set = function( _, value ) TitanPals.Settings.ShowMem = value; end,
                    },
                    tooltipbannor = {
                         type = "toggle", order = 2,
                         name = LB["TITAN_PALS_CONFIG_BANNOR"],
                         desc = LB["TITAN_PALS_CONFIG_BANNOR_DESC"],
                         get = function() return TitanPals.Settings.DisplayB; end,
                         set = function( _, value ) TitanPals.Settings.DisplayB = value; end,
                    },
		    tooltipshowoffline = {
                         type = "toggle", order = 4,
                         name = LB["TITAN_PALS_CONFIG_SHOWOFFLINE"],
                         desc = LB["TITAN_PALS_CONFIG_SHOWOFFLINE_DESC"],
                         get = function() return TitanPals.Settings.ShowOffline; end,
                         set = function( _, value ) TitanPals.Settings.ShowOffline = value; end,
                    },
                    tooltipnoalts = {
                         type = "toggle", order = 5, width = "full",
                         name = LB["TITAN_PALS_CONFIG_NOALTS"],
                         desc = LB["TITAN_PALS_CONFIG_NOALTS_DESC"],
                         get = function() return TitanPals.Settings.NoAlts; end,
                         set = function( _, value ) TitanPals.Settings.NoAlts = value; end,
                    },
                    debugmode = {
                         type = "toggle", order = 6,
                         name = LB["TITAN_PALS_CORE_DEBUG"],
                         desc = LB["TITAN_PALS_CORE_DEBUG_DESC"],
                         hidden = function() return not TitanPals_CoreUtils( "Author", ( "%s@%s" ):format( TitanPals.Profile.Player, TitanPals.Profile.Realm ) ) end,
                         get = function() return TitanPals.Debug.State; end,
                         set = function( _, value ) TitanPals.Debug.State = value; end,
                    },
               },
	  },
	  cat3 = {
               type = "group", order = 4,
               name = LB["TITAN_PALS_CONFIG_HEADER_TOOLTIP"],
               args = {
	            tooltippresets = {
                         type = "select", order = 1, width = "full",
                         name = LB["TITAN_PALS_CONFIG_PRESETS"],
                         desc = LB["TITAN_PALS_CONFIG_PRESETS_DESC"],
                         get = function() return TitanPals.Settings.TFormat; end,
                         set = function( _, value ) TitanPals.Settings.TFormat = value; end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( TitanPalsStaticArrays.presets ) do
                                   if ( v.value ~= TitanPals.Settings.TFormat ) then
                                        preList[v.value] = v.value;
                                   else
                                        preList[v.value] = ( "|cffffff9a%s|r" ):format( v.value );
                                   end
                              end
                              table.sort( preList, function( a, b )
                                   return string.lower( TitanPalsStaticArrays.persets[a].name ) < string.lower( TitanPalsStaticArrays.persets[b].name );
                              end );
                              return preList;
                         end
                    },
                    nulloption1 = {
                         type = "description", order = 2,
                         name = LB["TITAN_PALS_CONFIG_PRESETS_PREVIEW"],
                         cmdHidden = true,
                    },
               },
	  },
	  cat4 = {
               type = "group", order = 5,
               name = LB["TITAN_PALS_CONFIG_HEADER_ADD"],
               args = {
                    tooltipaddcustom = {
			 type = "input", order = 1, width = "full",
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP"],
			 desc = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP_DESC"],
			 get = function() return end,
			 set = function( _, value )
                              local tc = getn( TitanPalsStaticArrays.presets );
			      tc = ( tc + 1 );
			      local tName = ( "Style #%s" ):format( tc );
			      local tValue = value;
			      TitanPanelPals_AddNewPreset( tName, tValue );
			 end,
		    },
                    nulloptionct = {
			 type = "description", order = 2,
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP_INST"],
			 cmdHidden = true;
                    },
               },
	  },
          cat5 = {
               type = "group", order = 6,
               name = LB["TITAN_PALS_CONFIG_HEADER_REMOVE"],
	       args = {
	            tooltippresetsremove = {
                         order = 1, type = "select", width = "full",
                         name = LB["TITAN_PALS_CONFIG_PRESETS"],
                         desc = LB["TITAN_PALS_CONFIG_REMOVE_PRESETS_DESC"],
                         get = function() return TitanPalsFormatToRemove; end,
                         set = function( _, value ) 
                              TitanPals.Temp.TitanPalsFormatToRemove = value;
                              TitanPalsFormatToRemove = value; 
			 end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( TitanPalsStaticArrays.presets ) do
                                   if ( v.value ~= TitanPals.Settings.TFormat ) then
                                        preList[v.value] = ( "%s %s" ):format( v.name, v.value );
                                   else
                                        preList[v.value] = ( "|cffffff9a%s %s|r" ):format( v.name, v.value );
                                   end
                              end
                              if ( TitanPalsFormatToRemove ~= "None" ) then
                                   preList["None"] = LB["TITAN_PALS_CONFIG_NONE"];
                              else
                                   preList["None"] = ( "|cffffff9a%s|r" ):format( LB["TITAN_PALS_CONFIG_NONE"] );
                              end
                              return preList;
                         end
                    },
		    tooltippresetremovebutton = {
                         type = "execute", order = 2,
			 name = LB["TITAN_PALS_CONFIG_REMOVE"],
			 desc = LB["TITAN_PALS_CONFIG_REMOVE_DESC"],
                         disabled = function()
                              if ( TitanPalsFormatToRemove ~= LB["TITAN_PALS_CONFIG_NONE"] ) then
                                   return false;
                              else
                                   return true;
                              end
                         end,
                         func = function()
                              if ( TitanPalsFormatToRemove == LB["TITAN_PALS_CONFIG_NONE"] ) then return; end
                              -- Need to add a check to see if the TFormat is about to be removed and if so change the TFormat to default
                              local k, v, z;
                              for k, v in pairs( TitanPalsStaticArrays.presets ) do
                                   if ( TitanPals.Settings.TFormat == TitanPalsFormatToRemove and not z ) then 
                                        TitanPals.Settings.TFormat = v.value;
                                        z = true;
                                   end
                                   if ( v.value == TitanPalsFormatToRemove ) then
                                        table.remove( TitanPalsStaticArrays.presets, k );
                                        TitanPalsFormatToRemove = LB["TITAN_PALS_CONFIG_NONE"];
                                        break;
                                   end
                              end
                         end,
		    },
               },
          },
	  cat6 = {
	       type = "group", order = 6,
               name = LB["TITAN_PALS_CONFIG_HEADER_STATUS"],
               args = {
                    selectt = {
                         type = "select", order = 1,
                         name = LB["TITAN_PALS_CONFIG_STATUS"],
                         desc = LB["TITAN_PALS_CONFIG_STATUS_DESC"],
                         get = function() return TitanPals.Settings.dDefault;  end,
                         set = function( _, value ) TitanPals.Settings.dDefault = value; end,
                         values = function()
                              local db = { { name = LB["TITAN_PALS_CONFIG_AWAY"], value = ( "<%s>" ):format( LB["TITAN_PALS_CONFIG_AWAY"] ), }, { name = LB["TITAN_PALS_CONFIG_DND"], value = ( "<%s>" ):format( LB["TITAN_PALS_CONFIG_DND"] ), }, };
                              local preList = {};
                              local v;
                              for _, v in pairs( db ) do
                                   if ( v.name ~= TitanPals.Settings.dDefalut ) then
                                        preList[v.name] = v.name;
                                   else
                                        preList[v.name] = ( "|cffffff9a%s|r" ):format( v.name );
                                   end
                              end
                              return preList;
                         end,
                    },
                    scat1 = {
                         type = "group", order = 2, inline = true,
                         name = LB["TITAN_PALS_CONFIG_BUSY"],
                         hidden = function()
                              if ( TitanPals.Settings.dDefault == LB["TITAN_PALS_CONFIG_AWAY"] ) then
                                   return false;
                              else
                                   return true;
                              end
                         end,
                         args = {
                              statustypes = {
                                   type = "select", order = 1, width = "full",
                                   name = LB["TITAN_PALS_CONFIG_STATUS_TYPES"],
                                   desc = LB["TITAN_PALS_CONFIG_STATUS_TYPES_DESC"],
                                   get = function() return TitanPals.Settings.SFormat; end,
                                   set = function( _, value ) TitanPals.Settings.SFormat = value; end,
                                   values = function()
                                        local preList = {};
                                        local v;
                                        for _, v in pairs( TitanPalsStaticArrays.sTypes ) do
                                             if ( v.value ~= TitanPals.Settings.SFormat ) then
                                                  preList[v.name] = v.value;
                                             else
                                                  preList[v.name] = ( "|cffffff9a%s|r" ):format( v.value );
                                             end
                                        end
                                        return preList;
                                   end
                              },
                              nulloptionct = {
                                   type = "description", order = 2,
                                   name = LB["TITAN_PALS_CONFIG_STATUS_TYPES_INST"],
                                   cmdHidden = true
                              },
                         },
                    },
                    scat2 = {
                         type = "group", order = 3, inline = true,
                         name = LB["TITAN_PALS_CONFIG_DNDLABEL"],
                         hidden = function()
                              if ( TitanPals.Settings.dDefault == LB["TITAN_PALS_CONFIG_DND"] ) then
                                   return false;
                              else
                                   return true;
                              end
                         end,
                         args = {
                              statustypes = {
                                   type = "select", order = 1, width = "full",
                                   name = LB["TITAN_PALS_CONFIG_DND_TYPES"],
                                   desc = LB["TITAN_PALS_CONFIG_DND_TYPES_DESC"],
                                   get = function() return TitanPals.Settings.DFormat; end,
                                   set = function( _, value ) TitanPals.Settings.DFormat = value; end,
                                   values = function()
                                        local preList = {};
                                        local v;
                                        for _, v in pairs( TitanPalsStaticArrays.dTypes ) do
                                             if ( v.value ~= TitanPals.Settings.DFormat ) then
                                                  preList[v.name] = v.value;
                                             else
                                                  preList[v.name] = ( "|cffffff9a%s|r" ):format( v.value );
                                             end
                                        end
                                        return preList;
                                   end
                              },
                              nulloptionct = {
                                   type = "description", order = 2,
                                   name = LB["TITAN_PALS_CONFIG_DND_TYPES_INST"],
                                   cmdHidden = true
                              },
                         },
                    },
               },
          },
          cat7 = {
	       type = "group", order = 7,
	       name =  LB["TITAN_PALS_CONFIG_HEADER_REALID"];
	       args = {
                    realidonoroff = {
			 name = LB["TITAN_PALS_CONFIG_REALID_ONOFF"],
			 desc = LB["TITAN_PALS_CONFIG_REALID_ONOFF_DESC"],
                         type = "toggle", order = 1, width = "full",
			 get = function() return TitanPals.Settings.ShowBNTooltip; end,
			 set = function( _, value ) 
                              TitanPals.Settings.ShowBNTooltip = value;
                              TitanPanelButton_UpdateButton( LB["TITAN_PALS_CORE_ID"] );
                         end,
		    },
		    tooltipacustomrealid = {
			 type = "select", order = 2, width = "full",
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP_REALID"],
			 desc = LB["TITAN_PALS_CONFIG_CUSTOM_TOOLTIP_REALID_DESC"],
			 get = function() return TitanPals.Settings.RealIDFormat end,
			 set = function( _, value ) TitanPals.Settings.RealIDFormat = value; end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( TitanPalsStaticArrays.realidformats ) do
                                   if ( v.value ~= TitanPals.Settings.RealIDFormat ) then
                                        preList[v.value] = v.value;
                                   else
                                        preList[v.value] = ( "|cffffff9a%s|r" ):format( v.value );
                                   end
                              end
                              return preList;
                         end
		    },
                    nulloption1 = {
                         type = "description", order = 3,
                         name = "\n",
                         cmdHidden = true,
                    },
                    confrealinondesc = {
                         type = "description", order = 4,
                         name = LB["TITAN_PALS_CONFIG_REALID_NOTE1_ON"],
                         cmdHidden = true,
                    },
                    nulloption2 = {
                         type = "description", order = 5,
                         name = "\n",
                         cmdHidden = true,
                    },
                    confrealidoffdesc = {
                         type = "description", order = 6,
                         name = LB["TITAN_PALS_CONFIG_REALID_NOTE1_OFF"],
                         cmdHidden = true,
                    },
               },
	  },
	  cat8 = {
               type = "group", order = 8,
               name = LB["TITAN_PALS_CONFIG_HEADER_REALID_ADD"],
               args = {
                    tooltipaddrealidcustom = {
			 type = "input", order = 1, width = "full",
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_REALID_TOOLTIP"],
			 desc = LB["TITAN_PALS_CONFIG_CUSTOM_REALID_TOOLTIP_DESC"],
			 get = function() return end,
			 set = function( _, value )
                              local tc = getn( TitanPalsStaticArrays.realidformats );
			      tc = ( tc + 1 );
			      local tName = ( "Format #%s" ):format( tc );
			      local tValue = value;
			      TitanPanelPals_AddNewRealIDPreset( tName, tValue );
			 end,
		    },
                    nulloptionct = {
			 type = "description", order = 2,
			 name = LB["TITAN_PALS_CONFIG_CUSTOM_REALID_TOOLTIP_INST"],
			 cmdHidden = true,
                    },
               },
	  },
          cat9 = {
               type = "group", order = 9,
               name = LB["TITAN_PALS_CONFIG_HEADER_REALID_REMOVE"],
	       args = {
	            tooltippresetsremove = {
                         type = "select", order = 1, width = "full",
                         name = LB["TITAN_PALS_CONFIG_REALID_PRESETS"],
                         desc = LB["TITAN_PALS_CONFIG_REALID_REMOVE_PRESETS_DESC"],
                         get = function() return TitanPalsFormatToRemove; end,
                         set = function( _, value ) 
                              TitanPals.Temp["TitanPalsFormatToRemove"] = value;
                              TitanPalsFormatToRemove = value;
			 end,
                         values = function()
                              local preList = {};
                              local v;
                              for _, v in pairs( TitanPalsStaticArrays.realidformats ) do
                                   if ( v.value ~= TitanPals.Settings.RealIDFormat ) then
                                        preList[v.value] = ( "%s %s" ):format( v.name, v.value );
                                   else
                                        preList[v.value] = ( "|cffffff9a%s %s|r" ):format( v.name, v.value );
                                   end
                              end
                              if ( TitanPalsFormatToRemove ~= "None" ) then
                                   preList["None"] = LB["TITAN_PALS_CONFIG_NONE"];
                              else
                                   preList["None"] = ( "|cffffff9a%s|r" ):format( LB["TITAN_PALS_CONFIG_NONE"] );
                              end
                              return preList;
                         end,
                    },
		    tooltippresetremovebutton = {
                         type = "execute", order = 2,
			 name = LB["TITAN_PALS_CONFIG_REALID_REMOVE"],
			 desc = LB["TITAN_PALS_CONFIG_REALID_REMOVE_DESC"],
                         disabled = function()
                              if ( TitanPalsFormatToRemove ~= LB["TITAN_PALS_CONFIG_NONE"] ) then
                                   return false;
                              else
                                   return true;
                              end
                         end,
                         func = function()
                              if ( TitanPalsFormatToRemove == LB["TITAN_PALS_CONFIG_NONE"] ) then return; end
                              -- Need to add a check to see if the RealIDFormat is about to be removed and if so change the RealIDFormat to default
                              local k, v, z;
                              for k, v in pairs( TitanPalsStaticArrays.realidformats ) do
                                   if ( TitanPals.Settings.RealIDFormat == TitanPalsFormatToRemove and not z ) then 
                                        TitanPals.Settings.RealIDFormat = v.value;
                                        z = true;
                                   end
                                   if ( v.value == TitanPalsFormatToRemove ) then
                                        table.remove( TitanPalsStaticArrays.realidformats, k );
                                        TitanPalsFormatToRemove = LB["TITAN_PALS_CONFIG_NONE"];
                                        break;
                                   end
                              end
                         end,
		    },
               },
          },
          cat10 = {
               type = "group", order = 10,
               name = LB["TITAN_PALS_CONFIG_FRIENDEDIT"],
               hidden = function()
                    eCheck = TitanPals_CoreUtils( "checkEdit", "_false" );
                    return not eCheck;
               end,
               args = {
                    toonName = {
                         type = "select", order = 1, -- width = "full",
                         name = LB["TITAN_PALS_CONFIG_FRIEND"],
                         desc = LB["TITAN_PALS_CONFIG_FRIEND_DESC"],
                         get = function() return TitanPals.toonEdit.toonName; end,
                         set = function( _, value ) TitanPals.toonEdit.toonName = value; end,
                         values = function()
                              local list, labels = {}, {};
                              local _, labels = TitanPals_CoreUtils( "checkEdit", "_true" );
                              tinsert( labels, { name = LB["TITAN_PALS_CONFIG_SELECT"], value = LB["TITAN_PALS_CONFIG_SELECT"], } )
                              for _, v in pairs( labels ) do
                                   list[v.value] = v.name;
                              end
                              return list;
			 end,
                    },
                    toonClass = {
                         type = "select", order = 2, -- width = "full",
                         name = LB["TITAN_PALS_CONFIG_CLASS"],
                         desc = LB["TITAN_PALS_CONFIG_CLASS_DESC"],
                         get = function() return TitanPals.toonEdit.toonClass; end,
                         set = function( _, value ) TitanPals.toonEdit.toonClass = value; end,
                         disabled = function()
                              if ( TitanPals.toonEdit.toonName ~= LB["TITAN_PALS_CONFIG_SELECT"] ) then
                                   return false;
                              else
                                   return true;
                              end
                         end,
                         values = function()
                              TitanPals.Temp = {};
                              local list, labels = {}, {};
                              local cClass, fClass, lClass, sClass;
                              local table1, table2, table3 = TitanPals_CoreUtils( "getLocals" )
                              for name, class in pairs( TitanPalsToEng ) do
                                   lClass = table1[class];
                                   fClass = table2[class];
                                   cClass = fClass:format( lClass )
                                   tinsert( labels, { color = cClass, class = lClass, } )                                   
                              end
                              tinsert( labels, { color = LB["TITAN_PALS_CONFIG_UNKNOWN"], class = LB["TITAN_PALS_CONFIG_UNKNOWN"] } );
                              for _, v in pairs( labels ) do
                                   list[v.class] = v.color;
                              end
                              return list;
			 end,
                    },
                    toonLevel = {
                         type = "range", order = 3,
                         name = LB["TITAN_PALS_CONFIG_LEVEL"],
                         desc = LB["TITAN_PALS_CONFIG_LEVEL_DESC"],
                         get = function() return TitanPals.toonEdit.toonLevel; end,
                         set = function( _, value ) TitanPals.toonEdit.toonLevel = tonumber( value ); end,
			 min = LB["TITAN_PALS_CORE_MINLEVEL"],
			 max = LB["TITAN_PALS_CORE_MAXLEVEL"],
                         step = 1,
                         disabled = function()
                              if ( TitanPals.toonEdit.toonClass ~= LB["TITAN_PALS_CONFIG_UNKNOWN"] ) then
                                   return false;
                              else
                                   return true;
                              end
                         end,
                    },
                    toonAlt = {
                         type = "select", order = 4, -- width = "full",
                         name = LB["TITAN_PALS_CONFIG_ALTS"],
                         desc = LB["TITAN_PALS_CONFIG_ALTS_DESC"],
                         get = function() return TitanPals.toonEdit.toonAlt; end,
                         set = function( _, value ) TitanPals.toonEdit.toonAlt = value; end,
                         disabled = function()
                              if ( TitanPals.toonEdit.toonLevel ~= 0 ) then
                                   return false;
                              else
                                   return true;
                              end
                         end,
                         values = function()
                              local list = {};
                              local labels = { { name = LB["TITAN_PALS_CONFIG_SELECT"], value = LB["TITAN_PALS_CONFIG_SELECT"], }, { name = LB["TITAN_PALS_CONFIG_YES"], value = LB["TITAN_PALS_CONFIG_YES"], }, { name = LB["TITAN_PALS_CONFIG_NO"], value = LB["TITAN_PALS_CONFIG_NO"], }, };
                              for _, v in pairs( labels ) do
                                   list[v.name] = v.value;
                              end
                              return list;
			 end,
                    },
                    toonSpacer = {
                         type = "description", order = 5,
                         name = "\n",
                         cmdHidden = true,
                    },
                    toonSet = {
                         type = "execute", order = 6,
                         name = LB["TITAN_PALS_CONFIG_EDITCHAR"],
                         desc = LB["TITAN_PALS_CONFIG_EDITCHAR_DESC"],
                         disabled = function()
                              local dStatus
                              if ( TitanPals.toonEdit.toonName ~= LB["TITAN_PALS_CONFIG_SELECT"] and TitanPals.toonEdit.toonClass ~= LB["TITAN_PALS_CONFIG_UNKNOWN"] and TitanPals.toonEdit.toonName ~= "0" and TitanPals.toonEdit.toonAlt ~= LB["TITAN_PALS_CONFIG_SELECT"] ) then
                                   dStatsus = false;
                              else
                                   dStatus = true;
                              end
                              return dStatus;
                         end,
                         func = function()
                              if ( TitanPals.toonEdit.toonName ~= LB["TITAN_PALS_CONFIG_SELECT"] ) then
                                   if ( TitanPals.toonEdit.toonClass ~= LB["TITAN_PALS_CONFIG_UNKNOWN"] and TitanPals.toonEdit.toonLevel ~= 0 ) then
                                        local name = TitanPals.toonEdit.toonName;
                                        local class = TitanPals.toonEdit.toonClass;
                                        local level = TitanPals.toonEdit.toonLevel;
                                        TitanPalsLists[palsDB][name].Class = class;
                                        TitanPalsLists[palsDB][name].Level = tonumber( level );
                                        if ( TitanPals.toonEdit.toonAlt == LB["TITAN_PALS_CONFIG_YES"] ) then
                                             TitanPalsAlts[palsDB][name] = class;
                                        end
                                        TitanPals.toonEdit = {
                                             toonName = LB["TITAN_PALS_CONFIG_SELECT"],
                                             toonClass = LB["TITAN_PALS_CONFIG_UNKNOWN"],
                                             toonLevel = tonumber( 0 ),
                                             toonAlt = LB["TITAN_PALS_CONFIG_SELECT"],
                                        };
                                        collectgarbage( "collect" );
                                   end
                              end
                         end,
                    },
               },
          },
          cat11 = {
               type = "group", order = 11,
               name = LB["TITAN_PALS_CORE_DEBUG_HEADER"],
               hidden = function() return not TitanPals.Debug.State; end,
               args = {
                    event = {
                         type = "toggle", order = 1,
                         name = LB["TITAN_PALS_CORE_DEBUG_EVENT"],
                         desc = LB["TITAN_PALS_CORE_DEBUG_EVENT_DESC"],
                         get = function() return TitanPals.Debug.Event; end,
                         set = function( _, value ) TitanPals.Debug.Event = value; end,
                    },
                    logevent = {
                         type = "toggle", order = 2,
                         name = LB["TITAN_PALS_CORE_DEBUG_LOGEVENT"],
                         desc = LB["TITAN_PALS_CORE_DEBUG_LOGEVENT_DESC"],
                         get = function() return TitanPals.Debug.LogEvent; end,
                         set = function( _, value ) TitanPals.Debug.LogEvent = value; end,
                    },
		    playfriends = {
                         type = "execute", order = 3,
			 name = LB["TITAN_PALS_CORE_DEBUG_PRINT_FRIENDS"],
			 desc = LB["TITAN_PALS_CORE_DEBUG_PRINT_FRIENDS_DESC"],
                         func = function()
                              TitanPals_CoreUtils( "playFriends" );
                         end,
		    },
		    playalts = {
                         type = "execute", order = 4,
			 name = LB["TITAN_PALS_CORE_DEBUG_PRINT_ALTS"],
			 desc = LB["TITAN_PALS_CORE_DEBUG_PRINT_ALTS_DESC"],
                         func = function()
                              TitanPals_CoreUtils( "playAlts" );
                         end,
		    },

		    remauth = {
                         type = "execute", order = 5,
			 name = LB["TITAN_PALS_CORE_DEBUG_REMAUTH"],
                         hidden = function()
                              if ( getn( TitanPals.AddAuth ) == 0 ) then
                                   return true;
                              else
                                   return false;
                              end
                         end,
			 desc = function()
                              local author = ( "%s@%s" ):format( TitanPals.Profile.Player, TitanPals.Profile.Realm )
                              return LB["TITAN_PALS_CORE_DEBUG_REMAUTH_DESC"]:format( author )
                         end,
                         func = function()
                              TitanPals_CoreUtils( "RemAuth" );
                              TitanPals.Debug.State = false;
                              TitanPals.Debug.Event = false;
                              TitanPals.Debug.LogEvent = false;
                         end,
		    },

               },
          },
     },
}

-- **************************************************************************
-- NAME : TitanPanelPals_AddNewPreset()
-- DESC : This will add a custom tooltip preset
-- **************************************************************************
function TitanPanelPals_AddNewPreset( presetname, presetvalue )
     local found
     for _,i in pairs( TitanPalsStaticArrays.presets ) do
          if i.name == presetname or i.value == presetvalue then
               found = true; break;
          end
     end	
     if ( not found ) then table.insert( TitanPalsStaticArrays.presets, { name = presetname, value = presetvalue } ); end
end

-- **************************************************************************
-- NAME : TitanPanelPals_AddNewRealIDPreset()
-- DESC : This will add a custom tooltip preset for RealID Display
-- **************************************************************************
function TitanPanelPals_AddNewRealIDPreset( presetname, presetvalue )
     local found
     for _, i in pairs( TitanPalsStaticArrays.realidformats ) do
          if ( i.name == presetname or i.value == presetvalue ) then
               found = true;
               break;
          end
     end	
     if ( not found ) then table.insert( TitanPalsStaticArrays.realidformats, { name = presetname, value = presetvalue } ); end
end

-- **************************************************************************
-- NAME : TitanPanelPalsButton_OnLoad()
-- DESC : Registers the plugin upon it loading
-- **************************************************************************
function TitanPanelPalsButton_OnLoad( self )
     self.registry = {
          id = LB["TITAN_PALS_CORE_ID"],
          builtIn = false,
          version = TITAN_VERSION,
          menuText = LB["TITAN_PALS_MENU_TEXT"],
          buttonTextFunction = "TitanPanelPalsButton_GetButtonText";
	  tooltipTitle = LB["TITAN_PALS_TOOLTIP"],
          tooltipCustomFunction = TitanPanelPals_SetTooltip;
          icon = LB["TITAN_PALS_CORE_ARTWORK"],
          iconWidth = 16,
          category = "Information",
          savedVariables = {
	       ShowIcon = 1,
	       ShowLabelText = 1,
          }
     };
     self:RegisterEvent( "ADDON_LOADED" );
     self:RegisterEvent( "PLAYER_ENTERING_WORLD" );
     self:RegisterEvent( "BN_FRIEND_ACCOUNT_ONLINE" );
     self:RegisterEvent( "BN_FRIEND_ACCOUNT_OFFLINE" );
     self:RegisterEvent( "FRIENDLIST_UPDATE" );
     self:RegisterEvent( "IGNORELIST_UPDATE" );

     -- Titan Pals Config
     AceConfig:RegisterOptionsTable( PlugInName, PalsOpt );
     AceConfigDialog:AddToBlizOptions( PlugInName, LB["TITAN_PALS_ADDON_LABEL"] );
     -- TitanPals_Init( "init" );
end

-- **************************************************************************
-- NAME : SLASH_TITANPALS*
-- DESC : Slash command setup
-- **************************************************************************
SLASH_TITANPALS1 = "/titanpals";
SLASH_TITANPALS2 = "/tp";
SlashCmdList["TITANPALS"] = function( msg )
     TitanPals_CommandHandler( "titanpals", msg );
end

-- **************************************************************************
-- NAME : TitanPals_ShowHelp( cmd )
-- DESC : Slash command help display
-- **************************************************************************
function TitanPals_ShowHelp( cmd )
     TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_LINE1"] );
     TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_LINE2"]:format( cmd ) );
     TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_LINE3"]:format( cmd ) );
     TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_LINE4"]:format( cmd ) );
     TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_LINE5"]:format( cmd ) );
     local auth = TitanPals_CoreUtils( "Author", ( "%s@%s" ):format( TitanPals.Profile.Player, TitanPals.Profile.Realm ) )
     if ( auth ) then
          TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_ADDAUTH"]:format( cmd ) );
          TitanPals_DebugUtils( "info", "help", LB["TITAN_PALS_HELP_REMAUTH"]:format( cmd ) );
     end
end

-- **************************************************************************
-- NAME : TitanPals_CommandHandler()
-- DESC : Slash command handler
-- **************************************************************************
function TitanPals_CommandHandler( slash, command )
     local _, _, cmd1, cmd2 = strfind( command, "^([^ ]+) (.+)$" );
     if ( not cmd1 ) then cmd1 = command; end
     --if ( not cmd1 ) then cmd1 = ""; end
     --if ( not cmd2 ) then cmd2 = ""; end
     if ( ( cmd1 == "" ) or ( cmd1 == "help" ) ) then
          TitanPals_ShowHelp( slash );
     elseif ( cmd1 == "options" or cmd1 == "config" ) then
          InterfaceOptionsFrame_OpenToCategory( LB["TITAN_PALS_ADDON_LABEL"] );
     elseif ( cmd1 == "reset" ) then
          TitanPals_ResetUtils( true, cmd1 );
     elseif ( cmd1 == "checkDB" or cmd1 == "checkdb" ) then
          TitanPals_ResetUtils( false, cmd1 );
     elseif ( cmd1 == "AddAuth" or cmd1 == "addauth" ) then
          TitanPals_CoreUtils( cmd1 );
     elseif ( cmd1 == "RemAuth" or cmd1 == "remauth" ) then
          TitanPals_CoreUtils( cmd1 );
     elseif ( cmd1 == "Test" or cmd1 == "test" ) then
          TitanPals.Settings.Version = nil;
          TitanPals.Settings.ArrayVersion = nil;
	  TitanPalsTest();
          collectgarbage( "collect" );
     elseif ( cmd1 == "Test2" or cmd1 == "test2" ) then
          TitanPalsTest()
          TitanPals_CheckUpdate()
     elseif ( cmd1 == "ver" or cmd1 == "version" ) then
          TitanPals_DebugUtils( "info", "VerCall", ( "Your are using addon version |cff00ff00%s|r" ):format( TitanPals.Settings.Version ) );
	  TitanPals_DebugUtils( "info", "VerCall", ( "Data Array version |cff00ff00%s|r" ):format( TitanPals.Settings.ArrayVersion ) );
	  TitanPals_DebugUtils( "info", "VerCall", ( "Update addon version |cff00ff00%s|r" ):format( LB["TITAN_PALS_CORE_VERSION"] ) );
	  TitanPals_DebugUtils( "info", "VerCall", ( "Update data array version |cff00ff00%s|r" ):format( LB["TITAN_PALS_CORE_DBVER"] ) );
     else
          TitanPals_DebugUtils( "error", "CommandHandler", LB["TITAN_PALS_HELP_ERROR"]:format( cmd1 ) )
     end
end
function TitanPalsTest()
     TitanPals.Settings.Version = tonumber( 73 );
     TitanPals.Settings.ArrayVersion = tonumber( 1 );
     TitanPals.Alts = {};
     TitanPals.Lists = {};
     TitanPals_DebugUtils( "info", "TitanPalsTest", ( "TitanPals.Settings.Version = %s" ):format( TitanPals.Settings.Version ) )
     TitanPals_DebugUtils( "info", "TitanPalsTest", ( "TitanPals.Settings.Version = %s" ):format( TitanPals.Settings.ArrayVersion ) )
     TitanPals["dTypes"] = {
          {
               ["name"] = "<DND>",
               ["value"] = "<DND>",
          }, -- [1]
          {
               ["name"] = "<Busy>",
               ["value"] = "<Busy>",
          }, -- [2]
          {
               ["name"] = "[DND]",
               ["value"] = "[DND]",
          }, -- [3]
          {
               ["name"] = "[Busy]",
               ["value"] = "[Busy]",
          }, -- [4]
          {
               ["name"] = "(DND)",
               ["value"] = "(DND)",
          }, -- [5]
          {
               ["name"] = "(Busy)",
               ["value"] = "(Busy)",
          }, -- [6]
     }
     TitanPals["realidformats"] = {
          {
               ["name"] = "Format #1",
               ["value"] = "!cn",
          }, -- [1]
          {
               ["name"] = "Format #2",
               ["value"] = "!cn - !rn",
          }, -- [2]
          {
               ["name"] = "Format #3",
               ["value"] = "!sn/!cn",
          }, -- [3]
          {
               ["name"] = "Format #4",
               ["value"] = "!ln/!cn",
          }, -- [4]
          {
               ["name"] = "Format #5",
               ["value"] = "!fn/!cn",
          }, -- [5]
          {
               ["name"] = "Format #6",
               ["value"] = "!fn/!cn - !rn",
          }, -- [6]
          {          
               ["name"] = "Format #7",
               ["value"] = "!fn ( !cn )",
          }, -- [7]
          {
               ["name"] = "Format #8",
               ["value"] = "!cn ( !fn )",
          }, -- [8]
     }
     TitanPals["presets"] = {
          {
               ["name"] = "Style #1",
               ["value"] = "!s (!l !c) !p ~ !n !z",
          }, -- [1]
          {
               ["name"] = "Style #2",
               ["value"] = "!s (!l) !uc ~ !n !z",
          }, -- [2]
          {
               ["name"] = "Style #3",
               ["value"] = "!s (!l) !p - !c !z ~ !n",
          }, -- [3]
          {
               ["name"] = "Style #4",
               ["value"] = "!s (!l) !c - !p !z ~ !n",
          }, -- [4]
          {
               ["name"] = "Style #5",
               ["value"] = "!s (!l) !uc !z ~ !n",
          }, -- [5]
          {
               ["name"] = "Style #6",
               ["value"] = "!s !p (!l) - !c !n ~ !z",
          }, -- [6]
          {
               ["name"] = "Style #7",
               ["value"] = "!s !z (!l) - !c !n ~ !p",
          }, -- [7]
          {
               ["name"] = "Style #8",
               ["value"] = "!s !c (!l) !z !n ~ !p",
          }, -- [8]
     }
     TitanPals["sTypes"] = {
          {
               ["name"] = "<Away>",
               ["value"] = "<Away>",
          }, -- [1]
          {
               ["name"] = "<AFK>",
               ["value"] = "<AFK>",
          }, -- [2]
          {
               ["name"] = "<A>",
               ["value"] = "<A>",
          }, -- [3]
          {
               ["name"] = "(Away)",
               ["value"] = "(Away)",
          }, -- [4]
          {
               ["name"] = "(AFK)",
               ["value"] = "(AFK)",
          }, -- [5]
          {
               ["name"] = "(A)",
               ["value"] = "(A)",
          }, -- [6]
          {
               ["name"] = "[Away]",
               ["value"] = "[Away]",
          }, -- [7]
          {
               ["name"] = "[AFK]",
               ["value"] = "[AFK]",
          }, -- [8]
          {
               ["name"] = "[A]",
               ["value"] = "[A]",
          }, -- [9]
     }
end